@@ -141,7 +141,7 @@ make_extract_index <- function(v, vmx, name="i") {
 		e <- ext_from_rc(x, min(rc[,1]), max(rc[,1]), min(rc[,2]), max(rc[,2]))
 		crop(x, e)
 	} else {
-		e <- x@cpp$extractCell(i-1)
+		e <- x@ptr$extractCell(i-1)
 		messages(x, "extract")
 		e <- do.call(cbind, e)
 		colnames(e) <- names(x)
@@ -155,7 +155,7 @@ make_extract_index <- function(v, vmx, name="i") {
 
 .extract_cell_layer <- function(x, i, lyrs, drop) {
 	if (drop) {
-		e <- x@cpp$extractCell(i-1)
+		e <- x@ptr$extractCell(i-1)
 		messages(x, "extract")
 		e <- do.call(cbind, e)
 		colnames(e) <- names(x)
