@@ -43,14 +43,14 @@ setMethod("dots", signature(x="SpatVector"),
 #	cols <- out$cols
 #	if (is.null(cols)) cols = rep("black", n)
 
-#	g <- lapply(x@cpp$linesList(), function(i) { names(i)=c("x", "y"); i } )
+#	g <- lapply(x@ptr$linesList(), function(i) { names(i)=c("x", "y"); i } )
 
 #	g <- geom(x, df=TRUE)
 #	g <- split(g, g[,1])
 #	g <- lapply(g, function(x) split(x[,3:4], x[,2]))
 #	n <- length(g)
 
-	g <- x@cpp$linesList()
+	g <- x@ptr$linesList()
 	lty <- rep_len(lty, n)
 	lwd <- rep_len(lwd, n)
 	for (i in 1:n) {
@@ -119,7 +119,7 @@ setMethod("dots", signature(x="SpatVector"),
 #				# g[[i]][[1]] <- a
 #			}
 
-	g <- x@cpp$polygonsList()
+	g <- x@ptr$polygonsList()
 	if (is.null(out$leg$density)) {
 		for (i in seq_along(g)) {
 			for (j in seq_along(g[[i]])) {
@@ -293,16 +293,23 @@ setMethod("dots", signature(x="SpatVector"),
 	levs <- levels(fz)
 	nlevs <- length(levs)
 
-	cols <- out$cols
-	ncols <- length(cols)
-	if (nlevs < ncols) {
-		i <- trunc((ncols / nlevs) * 1:nlevs)
-		cols <- cols[i]
+#	cols <- out$cols
+#	ncols <- length(cols)
+#	if (nlevs < ncols) {
+#		i <- trunc((ncols / nlevs) * 1:nlevs)
+#		cols <- cols[i]
+#	} else {
+#		cols <- rep_len(cols, nlevs)
+#	}
+	
+	if (out$legend_type == "classes" && nlevs > length(out$cols)) {
+		# more classes than colors: cycle from beginning
+		cols_idx <- rep_len(out$cols, nlevs)
 	} else {
-		cols <- rep_len(cols, nlevs)
+		# else sample the colors
+		cols_idx <- trunc(seq(1, length(out$cols), length.out = nlevs))
 	}
-	out$cols <- cols
-	out$leg$fill <- cols
+	out$leg$fill <- out$cols <- out$cols[cols_idx]
 	out$legend_type <- "classes"
 
 	if (!is.null(out$leg$legend)) {
@@ -335,7 +342,7 @@ setMethod("dots", signature(x="SpatVector"),
 		if (!any(is.na(out$mar))) { graphics::par(mar=out$mar) }
 		plot(out$lim[1:2], out$lim[3:4], type="n", xlab="", ylab="", asp=out$asp, xaxs="i", yaxs="i", axes=FALSE, main="")
 		if (!is.null(out$background)) {
-			graphics::rect(out$lim[1], out$lim[3], out$lim[2], out$lim[4], col=out$background)
+			graphics::rect(out$lim[1], out$lim[3], out$lim[2], out$lim[4], col=out$background, border=TRUE)
 		}
 	}
 	if (isTRUE(out$blank)) return(out)
@@ -579,6 +586,9 @@ setMethod("dots", signature(x="SpatVector"),
 		out$legend_sort <- FALSE
 	} else {
 		out$uv <- unique(out$v)
+		if (is.factor(out$v)) {
+			out$uv <- levels(out$v)[levels(out$v) %in% out$uv]
+		}
 		out$legend_sort <- isTRUE(sort)
 		out$legend_sort_decreasing <- isTRUE(decreasing)
 	}
