@@ -177,7 +177,7 @@ SpatVector SpatVector::aggregate(bool dissolve) {
 
 #include "geodesic.h"
 
-void extend_line(const double &x1, const double &y1, const double &x2, const double &y2, double &x, double &y, const bool &geo, const double &distance, bool plus) {
+void extend_line(const double &x1, const double &y1, double &x2, double &y2, const bool &geo, const double &distance) {
 	if (geo) {
 		double a = 6378137.0;
 		double f = 1/298.257223563;
@@ -185,30 +185,31 @@ void extend_line(const double &x1, const double &y1, const double &x2, const dou
 		struct geod_geodesic g;
 		geod_init(&g, a, f);
 		geod_inverse(&g, y1, x1, y2, x2, &s12, &azi1, &azi2);		
-		geod_direct(&g, y2, x2, azi2, distance, &y, &x, &azi1);
+		geod_direct(&g, y2, x2, azi2, distance, &y2, &x2, &azi1);
 	} else {
 		double bearing;
 		double dx = x2 - x1;
+		double dy = y2 - y1;
 		if (dx == 0) {
 			if (y2 > y1) {
-				bearing = 0;
+				bearing = -M_PI / 2;
 			} else {
-				bearing = M_PI;				
+				bearing = M_PI / 2;				
 			}
 		} else {
-			bearing = atan((y2-y1)/dx);
+			bearing = atan(dy/dx);
 		}
-		if (plus) {
-			x = x2 + distance * sin(bearing);
-			y = y2 + distance * cos(bearing);
+		if (x2 > x1) {
+			x2 += distance * cos(bearing);
+			y2 += distance * sin(bearing);			
 		} else {
-			x = x2 - distance * sin(bearing);
-			y = y2 - distance * cos(bearing);			
+			x2 -= distance * cos(bearing);			
+			y2 -= distance * sin(bearing);						
 		}
 	}
 }
 
-SpatVector SpatVector::elongate(double length) {
+SpatVector SpatVector::elongate(double length, bool flat) {
 
 	SpatVector out = *this;
 	size_t n = size();
@@ -227,22 +228,15 @@ SpatVector SpatVector::elongate(double length) {
 		return out;
 	}
 
-	double x, y;
-	bool geo = is_lonlat();
+	bool geo = (!flat) && is_lonlat();
 	
 	for (size_t i=0; i<n; i++) {
 		for (size_t j=0; j < out.geoms[i].size(); j++) {
 			SpatPart p = out.geoms[i].parts[j];
-			size_t n = p.x.size();
-			if (n < 2) continue;
-			extend_line(p.x[1], p.y[1], p.x[0], p.y[0], x, y, geo, length, false);
-			p.x.insert(p.x.begin(), x);
-			p.y.insert(p.y.begin(), y);
-			
-			extend_line(p.x[n-1], p.y[n-1], p.x[n], p.y[n], x, y, geo, length, true);
-			p.x.push_back(x);			
-			p.y.push_back(y);
-			
+			size_t pn = p.x.size();
+			if (pn < 2) continue;
+			extend_line(p.x[1]   , p.y[1]   , p.x[0] , p.y[0] , geo, length);
+			extend_line(p.x[pn-2], p.y[pn-2], p.x[pn-1], p.y[pn-1], geo, length);		
 			out.geoms[i].parts[j] = p;
 		}
 		out.geoms[i].computeExtent();
@@ -277,6 +271,7 @@ SpatVectorCollection SpatVector::split(std::string field) {
 		v.srs = srs;
 		v.df = df.subset_rows(r);
 		out.push_back(v);
+		out.names[out.names.size()-1] = v.df.getSvalue(i, 0);
 	}
 	return out;
 }
@@ -607,7 +602,7 @@ void rotit(std::vector<double> &x, std::vector<double> &y, const double &x0, con
 SpatVector SpatVector::rotate(double angle, std::vector<double> x0, std::vector<double> y0) {
 	angle = -M_PI * angle / 180;
 	size_t n = size();
-	if ((x0.size() == 0) || (y0.size() == 0)) {
+	if (x0.empty() || y0.empty()) {
 		SpatVector out;
 		out.setError("no center of rotation provided");
 		return out;
